
CREATE TABLE sensors (
    sensorid                INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 101, INCREMENT BY 1),
    description             VARCHAR(254),
    -- A active, D disabled
    status                  CHAR(1) NOT NULL CHECK (status IN ('A', 'D')),

    ip_address              VARCHAR(128),

    username                VARCHAR(64) NOT NULL UNIQUE,
    password                VARCHAR(128) NOT NULL,
    password_reserve        VARCHAR(128),
    password_reserve_time   TIMESTAMP DEFAULT '9999-01-01 00:00:00',

    PRIMARY KEY(sensorid)
);

CREATE INDEX sensors_username ON sensors(username);

INSERT INTO sensors ( username, description, password, status ) VALUES ( 'test', 'Test Sensor', 'xNfwYD7TIC1+k/RkO/9b0/RavKA=', 'A' );

CREATE TABLE http_transactions (
    txid                    INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 1001, INCREMENT BY 1),
    sensorid                INTEGER NOT NULL REFERENCES sensors,
    sensor_txid             VARCHAR(128),
    webappid                VARCHAR(128),
    sessionid               VARCHAR(128),
    userid					VARCHAR(128),

    -- F forensic log, FC compressed forensic log
    data_type               CHAR(8) NOT NULL,
    data_source             VARCHAR(254) NOT NULL,
    data_offset             INTEGER NOT NULL DEFAULT 0,
    data_length             INTEGER NOT NULL,
    data_hash               VARCHAR(128),

    create_time             TIMESTAMP NOT NULL,
    tx_time                 TIMESTAMP NOT NULL,

    hostname                VARCHAR(254) NOT NULL,
    remote_addr             VARCHAR(128) NOT NULL,
    remote_host             VARCHAR(254),
    remote_port				INTEGER,
    local_user              VARCHAR(128),

    request_method          VARCHAR(128),
    request_uri             VARCHAR(10240),
    query_string            VARCHAR(10240),
    protocol                VARCHAR(128),

    request_content_length  INTEGER,
    request_content_type    VARCHAR(128),

    response_status         INTEGER NOT NULL,
    response_content_type   VARCHAR(128),

    referer                 VARCHAR(512),
    user_agent              VARCHAR(512),

    tx_duration             INTEGER NOT NULL,
    tx_request_duration     INTEGER,
    tx_processing_duration  INTEGER,
    tx_response_duration    INTEGER,

    is_valid                CHAR(1) NOT NULL,
    was_blocked             CHAR(1) NOT NULL,
    keep					CHAR(1) DEFAULT 'N',
    alert_severity          INTEGER NOT NULL,
    alert_message           VARCHAR(1024),

    PRIMARY KEY(txid)
);

CREATE INDEX http_transactions_sensorid ON http_transactions(sensorid);
CREATE INDEX http_transactions_tx_time ON http_transactions(tx_time);
CREATE INDEX http_transactions_remote_addr ON http_transactions(remote_addr);
CREATE INDEX http_transactions_alert_severity ON http_transactions(alert_severity);

CREATE TABLE alerts (
    alertid                 INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 1001, INCREMENT BY 1),
    sensorid                INTEGER NOT NULL REFERENCES sensors,

    -- 'HTTPTX' for HTTP transactions
    alert_type              VARCHAR(128),
    -- PK of the table where details are kept
    alert_source_id         INTEGER,

    src_ip                  VARCHAR(128),
    src_host                VARCHAR(128),
    src_port                INTEGER,
    src_user                VARCHAR(128),

    dst_ip                  VARCHAR(128) NOT NULL,
    dst_host                VARCHAR(128),
    dst_port                INTEGER,
    dst_user                VARCHAR(128),

    protocol                VARCHAR(64),

    create_time             TIMESTAMP NOT NULL,
    event_time              TIMESTAMP NOT NULL,

    alert_status            VARCHAR(32) NOT NULL,
    alert_resolution        VARCHAR(32),
    alert_severity          INTEGER NOT NULL,
    alert_message           VARCHAR(1024) NOT NULL,
    was_blocked             CHAR(1) NOT NULL,
    is_starred              CHAR(1) NOT NULL DEFAULT 'N',

    alert_id                VARCHAR(32),
    alert_revision          VARCHAR(32),
    
    category				VARCHAR(64) DEFAULT 'NOT_DETERMINED',
    comment					VARCHAR(1024),   

    -- e.g. hostname
    data1                   VARCHAR(254),
    -- e.g. URI
    data2                   VARCHAR(254),
    data3                   VARCHAR(254),

    PRIMARY KEY(alertid)
);

CREATE INDEX alerts_status ON alerts(alert_status);
CREATE INDEX alerts_sourceid ON alerts(alert_source_id);
CREATE INDEX alerts_resolution ON alerts(alert_resolution);
CREATE INDEX alerts_is_starred ON alerts(is_starred);
CREATE INDEX alerts_create_time ON alerts(create_time);
CREATE INDEX alerts_event_time ON alerts(event_time);

CREATE TABLE alert_reports (
    reportid                INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 101, INCREMENT BY 1),
    create_time             TIMESTAMP NOT NULL,
    send_time				TIMESTAMP NOT NULL,
    
    type             		VARCHAR(8),
    recipients				VARCHAR(128),

    PRIMARY KEY(reportid)
);

CREATE INDEX reports_create_time ON alert_reports(create_time);

CREATE TABLE users (
	userid					INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 101, INCREMENT BY 1),
	username				VARCHAR(64),
	password				VARCHAR(128),
	display					INTEGER DEFAULT 110,
	timezone				DECIMAL DEFAULT 0,
	timeformat				VARCHAR(64) DEFAULT 'kk:mm.S',
	dateformat				VARCHAR(64) DEFAULT 'yyyy-MM-dd',
    status                  CHAR(1) NOT NULL CHECK (status IN ('A', 'D')),
	
	PRIMARY KEY(userid)
);

CREATE INDEX users_username ON users(username);

INSERT INTO users (username, password, status) VALUES ('admin', 'f8gnMPUtGx7nfSDh8H2gaSCEpwk=', 'A');

CREATE TABLE logs (
	logid 					INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 101, INCREMENT BY 1),
	message					VARCHAR(1024),
	timestamp				TIMESTAMP NOT NULL,
	severity				INTEGER NOT NULL,
	is_new					CHAR(1) DEFAULT 'Y',
	
	PRIMARY KEY(logid)
);

CREATE INDEX logs_severity ON logs(severity);
CREATE INDEX logs_timestamp ON logs(timestamp);
CREATE INDEX logs_is_new ON logs(is_new);

CREATE TABLE scheduled_reports (
	reportid 					INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 101, INCREMENT BY 1),
	name						VARCHAR(128),
	author						VARCHAR(128),
	description					VARCHAR(1024),
	email_recipients			VARCHAR(512),
	schedule					VARCHAR(254),
	sections					VARCHAR(512),
	
	PRIMARY KEY(reportid)
);


CREATE TABLE scheduled_reports_sensors (
	reportid INTEGER NOT NULL REFERENCES scheduled_reports,
	sensorid INTEGER NOT NULL REFERENCES sensors,
	
	PRIMARY KEY(reportid, sensorid)
);

CREATE TABLE generated_reports (
	reportid 					INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 101, INCREMENT BY 1),	
	name						VARCHAR(128),
	filename					VARCHAR(254),
	create_time					TIMESTAMP NOT NULL,
	start_time					TIMESTAMP NOT NULL,
	end_time					TIMESTAMP NOT NULL,
	email_recipients			VARCHAR(512),
	type 						VARCHAR(32) NOT NULL DEFAULT 'Unknown',
	scheduled_reportid 			INTEGER REFERENCES scheduled_reports(reportid) ON DELETE SET NULL,

	PRIMARY KEY(reportid)	
);

CREATE TABLE product_info (
	product_name				VARCHAR(128),
	product_version				VARCHAR(128),
	product_edition				VARCHAR(128)
);

INSERT INTO product_info (
	product_name, product_version, product_edition
) VALUES (
	'ModSecurity Console', '1.0.3', 'Standard'
);

CREATE TABLE licences (
	licenceid					INTEGER NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 101, INCREMENT BY 1),
	licence						VARCHAR(4096),
	status						CHAR(1) DEFAULT 'A',

	PRIMARY KEY(licenceid)
);

CREATE INDEX licences_status ON licences(status);

INSERT INTO licences (
	licence
) VALUES (
	'LS0tLS1CRUdJTiBGSUxFLS0tLS0KDQpwcm9kdWN0TmFtZSA9IE1vZFNlY3VyaXR5IENvbnNvbGUN
CnByb2R1Y3RWZXJzaW9uID0gMS4wLjINCnByb2R1Y3RVcmkgPSBodHRwOi8vd3d3Lm1vZHNlY3Vy
aXR5Lm9yZy9wcm9qZWN0cy9jb25zb2xlLw0KcHJvZHVjdFZlbmRvciA9IEJyZWFjaCBTZWN1cml0
eQ0KcHJvZHVjdFZlbmRvclVyaSA9IGh0dHA6Ly93d3cuYnJlYWNoLmNvbQ0KDQpsaWNlbmNlRGVz
Y3JpcHRpb24gPSBNb2RTZWN1cml0eSBDb25zb2xlIFRyaWFsIExpY2VuY2UNCmxpY2VuY2VHZW5l
cmF0ZWRPbiA9IDIwMDcvMDEvMjUNCmxpY2VuY2VWYWxpZFVudGlsID0gMjAwNy8wOC8wMQ0KDQps
aWNlbmNlZU5hbWUgPSBwdWJsaWMNCmxpY2VuY2VlVXJpID0gDQpsaWNlbmNlZUVtYWlsID0gDQoN
CmFsbG93ZWRTZW5zb3JzID0gMw0KDQoKLS0tLS1CRUdJTiBTSUdOQVRVUkUtLS0tLQpNQzBDRlFD
RjN3V3VDeDJlTVVwREdNSU1TZ3cxMDBQeW1nSVVOUEZvalh1cmpTUkhLdGJWcWpEM3Era05vTnc9
DQotLS0tLUVORCBTSUdOQVRVUkUtLS0tLQo='
);

